# coding: utf-8
from django.http import HttpRequest

from educommon import ioc

from .manager import rbac


class PermsProxy(object):

    u"""Proxy-объект для проверки прав доступа.

    Экземпляр данного класса сохраняется в request.perms и предоставляет
    возможность проверять наличие у пользователя разрешений по их имени.
    """

    def __init__(self, request):
        assert isinstance(request, HttpRequest), type(request)

        self._request = request
        self._current_user = ioc.get('get_current_user')(request)
        self._cache = {}

    def __getitem__(self, perm_name):
        u"""Возвращает True, если пользователю доступно разрешение.

        :param basestring perm_name: Название разрешения.

        :rtype: bool
        """
        if self._current_user is None:
            return False

        if perm_name not in self._cache:
            self._cache[perm_name] = rbac.has_perm(
                self._current_user, perm_name
            )

        return self._cache[perm_name]

    def __contains__(self, perm_name):
        u"""Возвращает True, если пользователю доступно разрешение.

        :param basestring perm_name: Название разрешения.

        :rtype: bool
        """
        return self[perm_name]


class PermissionsMiddleware(object):

    u"""Добавляет в request прокси-объект perms для проверки прав доступа."""

    def process_request(self, request):
        request.perms = PermsProxy(request)
